;-----------------------------------------------------------------------------
; frank the fruit fly
;-----------------------------------------------------------------------------
# pmio.asm is the include file that defines hardware register names 
# the file is included in the tutorial zip file

	.include "pm_init.s"
	pm_header "fruit flyfun", IRQ_KEY_POWER | IRQ_MUSIC | IRQ_PRC_COPY, 0
	.include "pmrom_music.s"
	.include "pmio.asm"

	.ram	ticker		1
	.ram	ticker2		1
	.ram	ticker3		1
	.ram	wingsflap	1
	.ram	frankdir	1
	.ram 	buttonpress 1
	.ram	screennumber 1
	.ram    health 1
	.ram    health10  1
	.ram	sfx_select	1
	.ram	bgm_select	1
	.ram	prc_tick	1

	; Power button, shutdown PM
irq_key_power:
	cint CINT_SHUTDOWN

	; Act as VSync counter
irq_prc_copy:
	push hl
	mov hl, prc_tick
	inc [hl]
	pop hl
	rirq IRQ_PRC_COPY

;-----------------------------------------------------------------------------
; Title screen Setup
;-----------------------------------------------------------------------------
main:  
	xor a,a
	mov [ticker],a
	mov [ticker2],a
	mov [ticker3],a
	mov [buttonpress],a
	mov [frankdir],a

	mov a,0
	mov [bgm_select],a
	
	; Interrupts
	enable_irqs IRQ_KEY_POWER | IRQ_PRC_COPY	

	; Init sound engine and play music (BA = Master time, HL = RAM Ptr)
	mov ba, MAP_BASE
	mov hl, $00FF
	call pmmusic_init

	; Enable interrupt
	enable_mirq
	
	call music

    movb	b, 95
    movw	HL, TILEMAP+95
	
bglinear:
    movb	[HL], b   ### write the counter variable to the map
    dec		HL        ### decrease map address
    jdbnz       bglinear
	
    movw	sp,REG_BASE
    movw	nn,REG_BASE		
	
	mov		x1, titlescreenpic
	movw	[REG_BASE+REG_LCD_TILEBGMEM], x1

	mov		x1, player
    movw	[REG_BASE+REG_LCD_SPRITEMEM], x1
	
    movb	[nn+REG_EVENT2P],INT2P_KEYPAD
    movb	[nn+REG_EVENT2S],INT2S_KEY_POWER

    ; Activate all interrupts
    mov		flags, 0

    movb	[nn+REG_LCD_CNT0], LCD0_RENDER|LCD0_TILEBG|LCD0_MAP12x16|LCD0_SPRITES
    movb	[nn+REG_LCD_CNT1], LCD1_ENABLE|LCD1_REFRESH1

	mov nn,OAM
	mov [nn+0],44
	mov [nn+1],55
	mov [nn+2],0
	mov [nn+3],OAM_ENABLE

	mov nn,REG_BASE
	
	jmp title_screen
	
start_game_:
	mov a,0
	mov [buttonpress],a	
		
	jmp start_game
	
;-----------------------------------------------------------------------------
; Move Frank
;-----------------------------------------------------------------------------	
frank_right1:
	call frank_right11
	jmp frank_y_movement
	
frank_left1:
	call frank_left11
	jmp frank_y_movement
	
frank_up1:
	call frank_up
	jmp level_1
	
frank_down1:
	call frank_down11
	jmp level_1
	
;-----------------------------------------------------------------------------
; Title screen
;-----------------------------------------------------------------------------
title_screen:
		mov		a, [frankdir] 
		cmp		a, 5
		jz		title_screen_2

		call read_keys
		test a, KEY_A
		jnz start_game_

title_screen_2:
		mov		a, [OAM+0] 
		cmp		a, 109
		jz		start_game_now

		mov		a, [ticker]
		add		a, 1
		mov		[ticker], a
	
		mov		a, [ticker] 
		cmp		a, 255
		jl		move_frank___
	
		jmp title_screen
	
;-----------------------------------------------------------------------------
; Move Frank around.
;-----------------------------------------------------------------------------	
move_frank___:
	call move_frank
	jmp title_screen
		
move_frank:
		mov		a, [ticker2]
		add		a, 1
		mov		[ticker2], a
	
		mov		a, [ticker2] 
		cmp		a, 77
		jl		move_frank2
		
		ret
		
move_frank2:
		mov a,0
		mov [ticker2],a	
		
		mov		a, [ticker3]
		add		a, 1
		mov		[ticker3], a

		;call flap_wings

		mov		a, [ticker3] 
		cmp		a, 252
		jz		move_frank3
		
		ret

move_frank_111:
		mov		a, [ticker]
		add		a, 1
		mov		[ticker], a
	
		mov		a, [ticker] 
		cmp		a, 255
		jl		move_frank___2224234123424124

		ret

move_frank___2224234123424124:
		mov		a, [ticker2]
		add		a, 1
		mov		[ticker2], a
	
		mov		a, [ticker2] 
		cmp		a, 77
		jl		move_frank22
		
		ret
		
move_frank22:
		mov a,0
		mov [ticker2],a	
	
		mov		a, [ticker3]
		add		a, 1
		mov		[ticker3], a

		mov		a, [ticker3] 
		cmp		a, 252
		jz		move_frank_in_game

		ret	
		
move_frank_in_game:
		call flap_wings

		mov a,0
		mov [ticker3],a	

		call read_keys
		test a, KEY_RIGHT
		jnz frank_right11
		
		call read_keys
		test a, KEY_LEFT
		jnz frank_left1

frank_y_movement:
		call read_keys
		test a, KEY_UP
		jnz frank_up1
		
		call read_keys
		test a, KEY_DOWN
		jnz frank_down1	
		
		ret
	
move_frank3:
		call flap_wings

		mov a,0
		mov [ticker3],a	
	
		mov		a, [frankdir] 
		cmp		a, 5
		jz		frank_right	
	
		mov		a, [OAM+0] 
		cmp		a, 31
		jz		frank_down	
	
		mov		a, [OAM+1] 
		cmp		a, 29
		jz		frank_left
	
		mov		a, [OAM+0] 
		cmp		a, 80
		jz		frank_up

frank_right:
		mov		a, [OAM+0]
		add		a, 1
		mov		[OAM+0], a
	
		mov		a, [frankdir] 
		cmp		a, 5
		jz		frank_down_start	
	
		ret
	
frank_right11:
		mov		a, [OAM+0]
		add		a, 1
		mov		[OAM+0], a
	
		mov		a, [OAM+0] 
		cmp		a, 96
		jg		next_screen_
	
		ret
	
next_screen_:
		mov		a, [screennumber]
		add		a, 1
		mov		[screennumber], a	

		mov a,24
		mov [OAM+0],a	

		jmp next_screen
	
frank_up:
		mov		a, [OAM+1] 
		cmp		a, 17
		jz		nomove

		mov		a, [OAM+1]
		sub		a, 1
		mov		[OAM+1], a
	
nomove:
		ret
		
frank_left:
		mov		a, [OAM+0] 
		cmp		a, 5
		jz		frank_right	

frank_left11:
		mov		a, [OAM+0] 
		cmp		a, 24
		jz		nomove

		mov		a, [OAM+0]
		sub		a, 1
		mov		[OAM+0], a
	
		ret
		
frank_down_start:
		mov		a, [wingsflap] 
		cmp		a, 2
		jl		frank_down
		
		ret
		
frank_down:
		mov		a, [OAM+1]
		add		a, 1
		mov		[OAM+1], a
	
		mov		a, [OAM+1] 
		cmp		a, 55
		jz		frank_right	

		ret
		
frank_down11:
		mov		a, [OAM+1] 
		cmp		a, 72
		jz		nomove

		mov		a, [OAM+1]
		add		a, 1
		mov		[OAM+1], a
		ret

start_game_now:
		mov		x1, screen1
		movw	[REG_BASE+REG_LCD_TILEBGMEM], x1
	
		mov a,24
		mov [OAM+0],a		
	
		mov a,24
		mov [OAM+1],a	
		
		mov a,6
		mov [frankdir],a	
		
		mov a,0
		mov [ticker],a
		
	mov a,106
	mov [health],a

	mov a,106
	mov [health10],a

	mov hl, bgm_select
	mov a, 1
	cmp [hl], a
	jz nomove
	inc [hl]
	mov a, 8
	
	call music

;-----------------------------------------------------------------------------
; Level 1
;-----------------------------------------------------------------------------
level_1:
		call move_frank_111
		call show_health
		
		jmp level_1
;-----------------------------------------------------------------------------
; Flap Wings
;-----------------------------------------------------------------------------		
flap_wings:
		mov		a, [wingsflap]
		add		a, 1
		mov		[wingsflap], a

		mov		a, [wingsflap] 
		cmp		a, 2
		jg		wings_down

		mov a,1
		mov [OAM+2],a
		
		ret
		
wings_down:
		mov		a, [wingsflap] 
		cmp		a, 4
		jz		reset_wings

		mov a,0
		mov [OAM+2],a
		
		ret

reset_wings:
		mov a,0
		mov [wingsflap],a
		
		ret	
		
;-----------------------------------------------------------------------------
; Game Starting code/animation
;-----------------------------------------------------------------------------			
start_game:
		mov a,5
		mov [frankdir],a

		jmp title_screen

next_screen:
		jmp level_1

;-----------------------------------------------------------------------------	
; health meter
;-----------------------------------------------------------------------------	
show_health:
		mov x1, TILEMAP+72
		mov a, [health10]	
		mov [x1], a

		mov x1, TILEMAP+84
		mov a, [health]
		mov [x1], a	

		ret

;-----------------------------------------------------------------------------	
; music
;-----------------------------------------------------------------------------	
music:
	mov b, 0
	mov a, [bgm_select]
	shl a
	shl a
	mov hl, bgm_list+2
	add hl, ba
	mov b, [hl]
	dec hl
	dec hl
	mov hl, [hl]
	call pmmusic_playbgm
	
	ret

;-----------------------------------------------------------------------------
; Read Keypad Return A, 0 = Released, 1 = Pressed
;-----------------------------------------------------------------------------
read_keys:
	mov a, [n+KEY_PAD]
	push b
	mov b, [n+KEY_PAD]
	xor a, b
	mov a, b
	pop b
	jnz read_keys
	not a
	ret		

;-----------------------------------------------------------------------------
; Files it uses
;-----------------------------------------------------------------------------
	.align 16
	
bgm_list_last:	.db 2	

bgm_list:
	.dd music_test2
	.dd level_1_music
	
	PMMUSIC_ALIGN

	.include "level_1_music.asm"	
	.include "music_title2.asm"	

	pm_align_tiles
titlescreenpic:
	.incbin franktitle.bin	
screen1:
	.incbin frankscreen1.bin	

	pm_align_spr
player:
	.incbin franksprites.bin